.functionality-test-section table {
	border-collapse: collapse;
}

.functionality-test-section table tr {
	border: 1px solid #ddd;	
}

.functionality-test-section table tr td {
	padding: 5px 10px;
	vertical-align: top;
}

.functionality-test-section table tr td h2 {
	margin-bottom: 5px;
	margin-top: 30px;
}

/* The first tr will be the status (pass/fail) */
.functionality-test-section table tr:first-of-type {
	background: none;
	border: none;
	font-weight: bold;	
}

/* Highlight errors and warnings. */
.wp-list-table .error {
	color: #dc3232;
	font-weight: bold;
}
.wp-list-table .warning {
	color: #ffba00;
	font-weight: bold;
}
.wp-list-table .success {
	color: green;
	font-weight: bold;
}

.functionality-test-section tr.heading {
	border: 0px;
	background: transparent;
}

.functionality-test-section pre {
	white-space: pre-line;
}