.cf:before, .cf:after { content: " "; display: table; }
.cf:after { clear: both; }
.cf { *zoom: 1; } /* ie7 only */


.bunyad-meta { margin: 5px 0 15px 0; }
.bunyad-meta .option { 
	overflow: hidden;
	padding: 15px 5px;
	border-bottom: 1px solid #eee;
}

.bunyad-meta .option:hover { background: #f9f9f9; }

.bunyad-meta .option:last-child { border-bottom: 0; }

.bunyad-meta .label {
	width: 20%;
	float: left;
	display: block;
	max-width: 200px;
	font-weight: 600;
}

.bunyad-meta .field {
	float: left;
	width: 80%;
	padding-left: 10px;
	display: block;
	
	-webkit-box-sizing: border-box;
	-moz-box-sizing: border-box;
	box-sizing: border-box;
}

.bunyad-meta .description {
	margin-top: 7px;
	font-size: 90%;
}

.bunyad-meta .field textarea {
	max-width: 100%;
	margin: 3px 0;
	
	-webkit-box-sizing: border-box;
	-moz-box-sizing: border-box;
	box-sizing: border-box;
}

.radio-option label span { margin-left: 5px; }

/* form elements */
.color-picker-element { 
	display: none; 
	position: absolute; 
	background-color: #fcfcfc; 
	border-radius: 5px; -webkit-border-radius: 5px; -moz-border-radius: 5px;
	box-shadow: 0px 1px 4px 0 rgba(0,0,0, 0.2); -webkit-box-shadow: 0px 1px 4px 0 rgba(0,0,0, 0.2); -moz-box-shadow: 0px 1px 4px 0 rgba(0,0,0, 0.2);
	z-index: 1;
}

.mce_bunyad_shortcodesList {
	font-size: 14px;	
}

.image-upload {
	display: none;
	max-width: 350px;
	margin-top: 10px;
}

.image-upload.visible {
	display: block;
}

.image-upload img {
	max-width: 100%;
}

.remove-image, .image-type {
	display: none !important;
}

.remove-image.visible, .image-type.visible {
	display: inline-block !important;
}