.listing .dashicons.dashicons-portfolio {
	color: #c4963d;
}

.listing .dashicons.dashicons-media-default {
	color: #0073aa;
}

.listing a,
a.restore-db,
a.view-db,
a.load-browser,
.breadcrumbs a {
	cursor: pointer;
}

.breadcrumbs .dashicons.dashicons-admin-home {
	vertical-align: text-bottom;	
}

.file-actions > td {
	padding-left:35px;
}

/* Mimic postbox toggle arrows. These generally are in the last thead th. */
.bulk-action-notice .toggle-indicator {
	float: right;
	cursor: pointer;
}
.bulk-action-notice .toggle-indicator.closed:before {
	content: "\f140";
}