<?php
/**
 * Outputs the green Envira Gallery Header
 *
 * @since   1.5.0
 *
 * @package Envira_Gallery
 * @author 	David Bisset, Tim Carr
 */
?>
<div id="envira-header-temp"></div>
<div id="envira-header" class="envira-header">
	<h1 class="envira-logo" id="envira-logo">
		<img src="<?php echo $data['logo']; ?>" alt="<?php _e( 'Envira Gallery', 'envira-gallery' ); ?>" width="339" height="26" />
	</h1>
</div>