.control-panel-themes .customize-themes-notifications div.boldgrid-backup-protect-now,
.control-panel-themes .customize-themes-notifications div.boldgrid-backup-in-progress,
.control-panel-themes .customize-themes-notifications div.boldgrid-backup-complete,
.control-panel-themes .customize-themes-notifications div.boldgrid-backup-protected {
	margin: 0px 25px 25px 25px;
}